/*
Copyright 2023.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// EDIT THIS FILE!  THIS IS SCAFFOLDING FOR YOU TO OWN!
// NOTE: json tags are required.  Any new fields you add must have json tags for the fields to be serialized.

// PostgreSQLSpec defines the desired state of PostgreSQL
type PostgreSQLSpec struct {
	// Version of Postgres to deploy. [16.0]
	Version string `json:"version"`
	// Username for authentication
	User string `json:"user"`
	// Password for authentication
	Password string `json:"password"`
	// Autogenerated name for default Database
	Database string `json:"database"`
}

// PostgreSQLStatus defines the observed state of PostgreSQL
type PostgreSQLStatus struct {
	Conditions Conditions `json:"conditions,omitempty"`
	Status     Status     `json:"status,omitempty"`
	ListenOn   Listen     `json:"listenOn,omitempty"`
}

type Listen struct {
	Node string `json:"node,omitempty"`
	Host string `json:"host,omitempty"`
	Port int32  `json:"port,omitempty"`
}

type Conditions struct {
	Pod     Status `json:"pod,omitempty"`
	Service Status `json:"service,omitempty"`
}

type Status string

const (
	Status_Error        Status = "error"
	Status_Pending      Status = "pending"
	Status_Initializing Status = "initializing"
	Status_NotReady     Status = "notReady"
	Status_Ready        Status = "ready"
)

func (a Status) IsLowerThan(b Status) bool {
	var m = map[Status]int{
		Status_Error:        0,
		Status_Pending:      1,
		Status_Initializing: 2,
		Status_NotReady:     3,
		Status_Ready:        4,
	}
	return m[a] < m[b]
}

//+kubebuilder:object:root=true
//+kubebuilder:subresource:status
//+kubebuilder:printcolumn:JSONPath=".status.listenOn.node",name=Node,type=string
//+kubebuilder:printcolumn:JSONPath=".status.listenOn.host",name=Host,type=string
//+kubebuilder:printcolumn:JSONPath=".status.listenOn.port",name=Port,type=number
//+kubebuilder:printcolumn:JSONPath=".status.status",name=Status,type=string

// PostgreSQL is the Schema for the postgresqls API
type PostgreSQL struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   PostgreSQLSpec   `json:"spec,omitempty"`
	Status PostgreSQLStatus `json:"status,omitempty"`
}

//+kubebuilder:object:root=true

// PostgreSQLList contains a list of PostgreSQL
type PostgreSQLList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []PostgreSQL `json:"items"`
}

func init() {
	SchemeBuilder.Register(&PostgreSQL{}, &PostgreSQLList{})
}
